/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.cat.translation.core.omegat;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Plugin;
import org.omegat.core.data.LegacyTM;
import org.omegat.core.data.StringEntry;
import org.omegat.core.data.StringEntryWrapper;
import org.omegat.core.matching.FuzzyMatcherWrapper;
import org.omegat.core.matching.NearString;
import org.omegat.util.Language;
import org.omegat.util.StaticUtils;
import org.omegat.util.TMXReader;

import benten.cat.translation.core.BentenTmEngine;
import benten.cat.translation.core.BentenTmSearchResult;
import benten.cat.translation.core.CatTranslationCorePlugin;
import benten.cat.translation.core.tmx.AbstractTmxPlugin;
import blanco.xliff.BlancoXliffParser;
import blanco.xliff.valueobject.BlancoXliff;
import blanco.xliff.valueobject.BlancoXliffBody;
import blanco.xliff.valueobject.BlancoXliffFile;
import blanco.xliff.valueobject.BlancoXliffTransUnit;

public class DefaultTmEngine implements BentenTmEngine {
	private static final boolean IS_DEBUG = false;

	private static final String DEFAULT_TMX = "default tmx";

	private List<LegacyTM> legacyTmList = new ArrayList<LegacyTM>();

	private List<BentenTmSearchResult> nearList = new ArrayList<BentenTmSearchResult>();

	public DefaultTmEngine() {
	}

	public void load() {
		File dummyOmegatPrefs = new File(StaticUtils.getConfigDir() + "omegat.prefs");
		if (!dummyOmegatPrefs.exists()) {
			try {
				PrintWriter writer = new PrintWriter(dummyOmegatPrefs);
				writer.print("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
				writer.close();
			} catch (IOException e) {
				CatTranslationCorePlugin.log(e);
			}
		}
		String tmxDir = getTmxDirPath();
		loadTmx(new File(tmxDir));
	}

	public void loadTmx(File dirTmx) {
		List<LegacyTM> result = new ArrayList<LegacyTM>();

		final File[] fileTmxs = dirTmx.listFiles();
		if (fileTmxs == null) {
			CatTranslationCorePlugin.log("The pathname does not denote directory: " + dirTmx.toString());
			return;
		}

		try {
			for (int index = 0; index < fileTmxs.length; index++) {
				final File fileTmx = fileTmxs[index];
				if (fileTmx.isFile() == false) {
					continue;
				}
				if (fileTmx.getName().toLowerCase().endsWith(".tmx") == false) {
					// ファイルの拡張子が .tmx のものだけ処理します。
					continue;
				}

				final TMXReader reader = new TMXReader("UTF-8", new Language("en-US"), new Language("ja-JP"), false);
				reader.loadFile(fileTmx.getAbsolutePath(), true);
				LegacyTM legacyTm = loadLegacyTM(fileTmx.getName(), reader);
				result.add(legacyTm);
			}
			if (result.size() > 0)
				legacyTmList = result;
		} catch (IOException e) {
			CatTranslationCorePlugin.log(e);
		}
	}

	@SuppressWarnings("static-access")
	private String getTmxDirPath() {
		String result = null;
		IExtensionRegistry registry = Platform.getExtensionRegistry();
		IExtensionPoint point = registry.getExtensionPoint(CatTranslationCorePlugin.PLUGIN_ID + ".tmx");

		if (IS_DEBUG) {
			System.out.println(point);
		}

		if (point == null)
			return result;

		Map<String, AbstractTmxPlugin> tmxMap = new HashMap<String, AbstractTmxPlugin>();

		IExtension[] extensions = point.getExtensions();
		for (int i = 0; i < extensions.length; i++) {
			IConfigurationElement[] cfgElems = extensions[i].getConfigurationElements();
			for (int j = 0; j < cfgElems.length; j++) {
				IConfigurationElement cfgElem = cfgElems[j];
				try {
					String name = cfgElem.getAttribute("name");
					Object tmxPluginClass;
					tmxPluginClass = cfgElem.createExecutableExtension("class");
					if (tmxPluginClass instanceof Plugin) {
						tmxMap.put(name, (AbstractTmxPlugin) tmxPluginClass);
					}
				} catch (CoreException e) {
					CatTranslationCorePlugin.log(e);
				}
			}

		}

		AbstractTmxPlugin tmxPlugin = tmxMap.get(DEFAULT_TMX);
		if (tmxPlugin == null)
			return result;

		// FIXME ファイルの読み込み方
		URL entry = tmxPlugin.getDefault().getBundle().getEntry("/tmx");
		try {
			result = FileLocator.resolve(entry).getPath();
		} catch (IOException e) {
			CatTranslationCorePlugin.log(e);
		}
		CatTranslationCorePlugin.log("Directory contains TMX: " + result);

		return result;

	}

	private LegacyTM loadLegacyTM(final String tmxFileName, final TMXReader reader) {
		final int num = reader.numSegments();
		List<StringEntry> strEntryList = new ArrayList<StringEntry>(num);
		for (int i = 0; i < num; i++) {
			String src = reader.getSourceSegment(i);
			String trans = reader.getTargetSegment(i);

			StringEntryWrapper se = new StringEntryWrapper(src);
			se.setTranslation(trans);
			strEntryList.add(se);
		}

		return new LegacyTM(tmxFileName, strEntryList);
	}

	public void loadLocalTmFromXliff(File fileXliff) {
		List<LegacyTM> result = new ArrayList<LegacyTM>();

		BlancoXliff xliff = new BlancoXliffParser().parse(fileXliff);
		List<?> fileList = xliff.getFileList();

		for (int i = 0; i < fileList.size(); i++) {
			BlancoXliffFile file = (BlancoXliffFile) fileList.get(i);
			BlancoXliffBody body = file.getBody();
			List<?> tuList = body.getTransUnitList();

			List<StringEntry> tuEntry = new ArrayList<StringEntry>(tuList.size());
			for (int j = 0; j < tuList.size(); j++) {
				BlancoXliffTransUnit tu = (BlancoXliffTransUnit) tuList.get(j);
				if (tu.getSource() == null) {
					continue;
				}
				if (tu.getTarget() == null) {
					continue;
				}
				if (tu.getSource().equals(tu.getTarget().getTarget())) {
					continue;
				}

				StringEntryWrapper se = new StringEntryWrapper(tu.getSource());
				se.setTranslation(tu.getTarget().getTarget());
				tuEntry.add(se);
			}
			List<StringEntry> localTmEntry = new ArrayList<StringEntry>();
			localTmEntry.addAll(tuEntry);
			LegacyTM legacyTm = new LegacyTM(fileXliff.getName(), localTmEntry);
			result.add(legacyTm);
		}

		if (result.size() > 0)
			legacyTmList = result;
	}

	public List<BentenTmSearchResult> fuzzySearch(String target) {
		nearList.clear();
		final FuzzyMatcherWrapper fuzzyMatcher = FuzzyMatcherWrapper.getInstance();

		final StringBuffer targetStrBuf = new StringBuffer();
		if (target != null) {
			targetStrBuf.append(target);
		}

		final StringEntry tuEntry = new StringEntry(targetStrBuf.toString());

		List<NearString> nearStringList = fuzzyMatcher.match(tuEntry, legacyTmList);

		if (nearStringList.size() > 0) {
			if (IS_DEBUG) {
				System.out.println(nearStringList.size() + "件のマッチがみつかりました。");
				System.out.println("(原文)" + tuEntry.getSrcText());
			}

			for (int i = 0; i < nearStringList.size(); i++) {
				final NearString nearString = nearStringList.get(i);
				final BentenTmSearchResult result = new BentenTmSearchResult();
				result.setMatchQuality(Integer.toString(nearString.score) + "% ");
				result.setSource(nearString.str.getSrcText());
				result.setTarget(nearString.str.getTranslation());
				result.setOrigin(nearString.proj);
				nearList.add(result);

				if (IS_DEBUG) {
					System.out.println("------------------------------------");
					System.out.println((i + 1) + "件目");
					System.out.println(nearString.score + "%マッチ");
					System.out.println(nearString.str.getSrcText());
					System.out.println(nearString.str.getTranslation());
					System.out.println(nearString.proj);
				}
			}

			if (IS_DEBUG) {
				System.out.println("------------------------------------");
				System.out.println("");
			}
		}

		return nearList;

	}

	public void fuzzyMatchLocalTm(String target) {
		nearList.clear();
		final FuzzyMatcherWrapper fuzzyMatcher = FuzzyMatcherWrapper.getInstance();

		final StringBuffer targetStrBuf = new StringBuffer();
		if (target != null) {
			targetStrBuf.append(target);
		}
		final StringEntry tuEntry = new StringEntry(targetStrBuf.toString());

		final List<NearString> nearStringList = fuzzyMatcher.match(tuEntry, legacyTmList);

		if (nearStringList.size() > 0) {
			if (IS_DEBUG) {
				System.out.println(nearStringList.size() + "件のマッチがみつかりました。");
				System.out.println("(原文)" + tuEntry.getSrcText());
			}

			for (int i = 0; i < nearStringList.size(); i++) {
				final NearString nearString = nearStringList.get(i);
				final BentenTmSearchResult result = new BentenTmSearchResult();
				result.setMatchQuality(Integer.toString(nearString.score) + "% ");
				result.setSource(nearString.str.getSrcText());
				result.setTarget(nearString.str.getTranslation());
				result.setOrigin(nearString.proj);
				nearList.add(result);

				if (IS_DEBUG) {
					System.out.println("------------------------------------");
					System.out.println((i + 1) + "件目");
					System.out.println(nearString.score + "%マッチ");
					System.out.println(nearString.str.getSrcText());
					System.out.println(nearString.str.getTranslation());
				}
			}

			if (IS_DEBUG) {
				System.out.println("------------------------------------");
				System.out.println("");
			}
		}

	}

	public void unload() {
		// TODO 未実装!!!
	}
}
