/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.cat.core;

import java.io.File;
import java.io.IOException;

import benten.cat.translation.core.BentenTmDriverManager;
import benten.cat.translation.core.BentenTmEngine;
import benten.twa.cat.core.valueobject.BentenApplyExactMatchProcessInput;
import benten.twa.ui.AbstractTraverseDir;
import blanco.xliff.BlancoXliffParser;
import blanco.xliff.BlancoXliffSerializer;
import blanco.xliff.valueobject.BlancoXliff;
import blanco.xliff.valueobject.BlancoXliffFile;
import blanco.xliff.valueobject.BlancoXliffTarget;
import blanco.xliff.valueobject.BlancoXliffTransUnit;

public class BentenApplyExactMatchProcessImpl extends AbstractTraverseDir implements BentenApplyExactMatchProcess {

	/**
	 * Ant タスクの際にのみ利用される翻訳カウント。
	 */
	private int fTranslateCount = 0;

	/**
	 * この処理の入力オブジェクト。
	 */
	protected BentenApplyExactMatchProcessInput fInput;

	/**
	 * この翻訳で利用する翻訳メモリ。
	 */
	protected BentenTmEngine fTm;

	/**
	 * クラスをインスタンス化して処理を実行する際のエントリポイントです。
	 *
	 * @param input 処理の入力パラメータ。
	 * @return 処理の実行結果。
	 * @throws IOException 入出力例外が発生した場合。
	 * @throws IllegalArgumentException 入力値に不正が見つかった場合。
	 */
	public int execute(final BentenApplyExactMatchProcessInput input) throws IOException, IllegalArgumentException {
		fInput = input;
		if (input.getVerbose()) {
			System.out.println("twa: load tm: begin");
		}

		if (progress("翻訳メモリを読み込み中...")) {
			return 0;
		}

		// 翻訳メモリを読み込み。
		if (BentenTmDriverManager.getDrivers().length == 0) {
			throw new IllegalArgumentException("翻訳メモリが BentenTmDriverManager に 1 つも登録されていません。");
		}

		// TODO 1 つめの翻訳メモリのみ利用しています。複数対応してください。
		fTm = BentenTmDriverManager.getDrivers()[0].getEngineInstance();
		fTm.loadTmx(new File(input.getTmxdir()));

		if (input.getVerbose()) {
			System.out.println("twa: load tm: end");
		}

		if (progress("XLIFF ファイルを処理中...")) {
			return 0;
		}

		processDir(new File(input.getXliffdir()));

		if (input.getVerbose()) {
			System.out.println("  translated: " + fTranslateCount);
		}

		return 0;
	}

	/**
	 * 処理の中でアイテムが処理されるたびに進捗報告としてコールバックします。
	 *
	 * @param argProgressMessage 現在処理しているアイテムに関するメッセージ。
	 * @return 処理をそのまま継続する場合は false。処理中断をリクエストしたい場合は true。
	 */
	public boolean progress(String argProgressMessage) {
		return false;
	}

	@Override
	protected boolean canProcess(final File file) {
		return file.getName().toLowerCase().endsWith(".xlf");
	}

	@Override
	protected void processFile(final File targetXliffFile, final String baseDir) {
		if (fInput.getVerbose()) {
			System.out.println("twa: translate [" + targetXliffFile.getName() + "]: begin");
		}

		if (progress("ファイル [" + targetXliffFile.getName() + "] を処理中...")) {
			return;
		}

		final BlancoXliffParser parser = new BlancoXliffParser();

		// TODO 例外処理。
		final BlancoXliff xliff = parser.parse(targetXliffFile);

		int transUnitTotalCount = 0;
		for (BlancoXliffFile file : xliff.getFileList()) {
			transUnitTotalCount += file.getBody().getTransUnitList().size();
		}
		beginTask(transUnitTotalCount);

		int transUnitCount = 0;
		for (BlancoXliffFile file : xliff.getFileList()) {
			for (BlancoXliffTransUnit transUnit : file.getBody().getTransUnitList()) {
				if (progress("ファイル [" + targetXliffFile.getName() + "] (" + (++transUnitCount) + "/"
						+ transUnitTotalCount + ") を処理中...")) {
					break;
				}

				if (transUnit.getTranslate() == false) {
					continue;
				}
				if (transUnit.getSource() == null || transUnit.getSource().length() == 0) {
					continue;
				}
				if (transUnit.getTarget() != null && transUnit.getTarget().getTarget() != null
						&& transUnit.getTarget().getTarget().length() >= 0) {
					// target は既に存在しています。

					if (transUnit.getTarget().getState() != null
							&& transUnit.getTarget().getState().trim().length() > 0) {
						// state が設定されています。これは既に翻訳確定済みと判断します。
					} else {
						if (transUnit.getSource().equals(transUnit.getTarget().getTarget())) {
							// source と target とが一致しています。

							// state が無指定または空で、しかも source と target とが一致している場合にのみ、訳を上書きします。
							processTransUnit(transUnit);
							continue;
						}
					}

					// TODO target がすでに存在する場合、訳を上書きして良いのかどうか。
					// state 判断か?
					continue;
				}

				processTransUnit(transUnit);
				continue;
			}
		}

		if (progress("XLIFF ファイル [" + targetXliffFile.getName() + "] を作成／更新中...")) {
			return;
		}

		// TODO 例外処理
		new BlancoXliffSerializer().serialize(xliff, targetXliffFile);

		if (progress("XLIFF ファイル [" + targetXliffFile.getName() + "] を作成／更新しました。")) {
			return;
		}

		if (fInput.getVerbose()) {
			System.out.println("twa: translate [" + targetXliffFile.getName() + "]: end");
		}
	}

	/**
	 * trans-unit の完全一致訳を処理します。
	 * 
	 * @param transUnit
	 */
	protected void processTransUnit(final BlancoXliffTransUnit transUnit) {

		final String targetString = BentenTmUtil.searchExactMatch(fTm, transUnit.getSource());
		if (targetString == null) {
			// 完全一致訳はありませんでした。
			return;
		}

		// 完全一致訳が見つかりました。

		final BlancoXliffTarget target = new BlancoXliffTarget();
		transUnit.setTarget(target);

		// 完全一致訳をセットします。
		target.setTarget(targetString);
		// 完全一致を target にセットする場合には、state に translated をセットとします。
		target.setState("translated");

		fTranslateCount++;
	}
}
