/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.cat.core;

import java.io.File;
import java.io.IOException;
import java.util.List;

import benten.cat.translation.core.BentenTmDriverManager;
import benten.cat.translation.core.BentenTmEngine;
import benten.cat.translation.core.BentenTmSearchResult;
import benten.core.BentenConstants;
import benten.twa.cat.core.valueobject.BentenApplyFuzzyMatchProcessInput;
import benten.twa.ui.AbstractTraverseDir;
import blanco.xliff.BlancoXliffParser;
import blanco.xliff.BlancoXliffSerializer;
import blanco.xliff.BlancoXliffUtil;
import blanco.xliff.valueobject.BlancoXliff;
import blanco.xliff.valueobject.BlancoXliffAltTrans;
import blanco.xliff.valueobject.BlancoXliffFile;
import blanco.xliff.valueobject.BlancoXliffTarget;
import blanco.xliff.valueobject.BlancoXliffTool;
import blanco.xliff.valueobject.BlancoXliffTransUnit;

public class BentenApplyFuzzyMatchProcessImpl extends AbstractTraverseDir implements BentenApplyFuzzyMatchProcess {
	protected BentenTmEngine fTm;

	protected BentenApplyFuzzyMatchProcessInput fInput;

	public int execute(final BentenApplyFuzzyMatchProcessInput input) throws IOException, IllegalArgumentException {
		fInput = input;

		if (input.getVerbose()) {
			System.out.println("twa: load tm: begin");
		}

		if (progress("翻訳メモリを読み込み中...")) {
			return 0;
		}

		// 翻訳メモリを読み込み。
		if (BentenTmDriverManager.getDrivers().length == 0) {
			throw new IllegalArgumentException("翻訳メモリが BentenTmDriverManager に 1 つも登録されていません。");
		}

		// TODO 1 つめの翻訳メモリのみ利用しています。複数対応してください。
		fTm = BentenTmDriverManager.getDrivers()[0].getEngineInstance();
		fTm.loadTmx(new File(input.getTmxdir()));

		if (input.getVerbose()) {
			System.out.println("twa: load tm: end");
		}

		if (progress("XLIFF ファイルを処理中...")) {
			return 0;
		}

		processDir(new File(input.getXliffdir()));

		return 0;
	}

	/**
	 * 処理の中でアイテムが処理されるたびに進捗報告としてコールバックします。
	 *
	 * @param argProgressMessage 現在処理しているアイテムに関するメッセージ。
	 * @return 処理をそのまま継続する場合は false。処理中断をリクエストしたい場合は true。
	 */
	public boolean progress(String argProgressMessage) {
		return false;
	}

	@Override
	protected boolean canProcess(final File file) {
		return file.getName().toLowerCase().endsWith(".xlf");
	}

	@Override
	protected void processFile(final File targetXliffFile, final String baseDir) {
		if (progress("ファイル [" + targetXliffFile.getName() + "] を処理中...")) {
			return;
		}

		if (fInput.getVerbose()) {
			System.out.println("twa: translate [" + targetXliffFile.getName() + "]: begin");
		}

		final BlancoXliffParser parser = new BlancoXliffParser();

		// TODO 例外処理。
		final BlancoXliff xliff = parser.parse(targetXliffFile);

		int matchCount = 0;

		int transUnitTotalCount = 0;
		for (BlancoXliffFile file : xliff.getFileList()) {
			transUnitTotalCount += file.getBody().getTransUnitList().size();
		}
		beginTask(transUnitTotalCount);

		int transUnitCount = 0;
		for (BlancoXliffFile file : xliff.getFileList()) {
			// 有無を言わさず tool-id を設定します。
			setToolId(file);

			for (BlancoXliffTransUnit transUnit : file.getBody().getTransUnitList()) {
				if (progress("ファイル [" + targetXliffFile.getName() + "] (" + (++transUnitCount) + "/"
						+ transUnitTotalCount + ") を処理中...")) {
					break;
				}

				if (transUnit.getTranslate() == false) {
					continue;
				}
				if (transUnit.getSource() == null || transUnit.getSource().length() == 0) {
					continue;
				}

				// clear self alt-trans
				// TODO クリアするかどうかは、フラグで指定したい。
				clearSelfAltTrans(transUnit);

				// TODO 件数を Eclipse に報告したい。
				processTransUnit(transUnit, fTm);
				matchCount++;
				if (matchCount > 100) {
					// TODO 暫定的に中断。
					// break;
				}
			}
		}

		if (progress("XLIFF ファイル [" + targetXliffFile.getName() + "] を作成／更新中...")) {
			return;
		}

		// TODO 例外処理
		new BlancoXliffSerializer().serialize(xliff, targetXliffFile);

		if (progress("XLIFF ファイル [" + targetXliffFile.getName() + "] を作成／更新しました。")) {
			return;
		}

		if (fInput.getVerbose()) {
			System.out.println("twa: translate [" + targetXliffFile.getName() + "]: end");
		}
	}

	/**
	 * trans-unit を処理します。
	 * 
	 * @param transUnit
	 * @param tm
	 */
	protected void processTransUnit(final BlancoXliffTransUnit transUnit, final BentenTmEngine tm) {

		// あいまい翻訳を検索
		final List<BentenTmSearchResult> matchList = tm.fuzzySearch(transUnit.getSource());

		for (BentenTmSearchResult entry : matchList) {
			final BlancoXliffAltTrans altTrans = new BlancoXliffAltTrans();
			transUnit.getAltTransList().add(altTrans);

			altTrans.setToolId(BentenConstants.PROJECT_NAME_LOWER);
			altTrans.setOrigin(entry.getOrigin());
			altTrans.setMatchQuality(entry.getMatchQuality().trim());
			altTrans.setSource(entry.getSource());
			final BlancoXliffTarget target = new BlancoXliffTarget();
			altTrans.setTarget(target);
			altTrans.getTarget().setTarget(entry.getTarget());
		}
	}

	/**
	 * 自分で登録した alt-trans をクリアーします。
	 * 
	 * @param transUnit
	 */
	protected void clearSelfAltTrans(final BlancoXliffTransUnit transUnit) {
		for (int index = transUnit.getAltTransList().size() - 1; index >= 0; index--) {
			final BlancoXliffAltTrans altTrans = transUnit.getAltTransList().get(index);
			if (altTrans.getToolId().equals(BentenConstants.PROJECT_NAME_LOWER)) {
				// これは自分の訳です。
				// 除去してしまいます。
				transUnit.getAltTransList().remove(index);
			}
		}
	}

	/**
	 * tool-id を設定します。
	 * 
	 * @param file
	 */
	protected void setToolId(final BlancoXliffFile file) {
		final BlancoXliffTool tool = new BlancoXliffTool();
		tool.setToolId(BentenConstants.PROJECT_NAME_LOWER);
		tool.setToolName(BentenConstants.PROJECT_NAME);
		BlancoXliffUtil.setTool(file.getHeader(), tool);
	}
}
