/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.cat.core;

import java.io.File;
import java.io.IOException;
import java.util.List;

import benten.core.io.IniFile;
import benten.twa.cat.core.valueobject.BentenMarkNoTransableProcessInput;
import benten.twa.ui.AbstractTraverseDir;
import blanco.xliff.BlancoXliffParser;
import blanco.xliff.BlancoXliffSerializer;
import blanco.xliff.valueobject.BlancoXliff;
import blanco.xliff.valueobject.BlancoXliffFile;
import blanco.xliff.valueobject.BlancoXliffTransUnit;

public class BentenMarkNoTransableProcessImpl extends AbstractTraverseDir implements BentenMarkNoTransableProcess {
	protected BentenMarkNoTransableProcessInput fInput;
	protected final List<String> markNoTransableList;

	public BentenMarkNoTransableProcessImpl() {
		markNoTransableList = IniFile.asList(this, "conf/mark-no-transable.ini");
	}

	public int execute(final BentenMarkNoTransableProcessInput input) throws IOException, IllegalArgumentException {
		fInput = input;
		processDir(new File(input.getXliffdir()));
		return 0;
	}

	/**
	 * 処理の中でアイテムが処理されるたびに進捗報告としてコールバックします。
	 *
	 * @param argProgressMessage
	 *            現在処理しているアイテムに関するメッセージ。
	 * @return 処理をそのまま継続する場合は false。処理中断をリクエストしたい場合は true。
	 */
	public boolean progress(String argProgressMessage) {
		return false;
	}

	@Override
	protected boolean canProcess(final File file) {
		return file.getName().toLowerCase().endsWith(".xlf");
	}

	@Override
	protected void processFile(final File fileCurrent, final String baseDir) {
		if (progress("ファイル [" + fileCurrent.getName() + "] を処理中...")) {
			return;
		}

		// TODO 例外処理
		final BlancoXliff xliff = new BlancoXliffParser().parse(fileCurrent);

		int transUnitTotalCount = 0;
		for (BlancoXliffFile file : xliff.getFileList()) {
			transUnitTotalCount += file.getBody().getTransUnitList().size();
		}
		beginTask(transUnitTotalCount);

		int transUnitCount = 0;
		for (BlancoXliffFile file : xliff.getFileList()) {
			for (BlancoXliffTransUnit transUnit : file.getBody().getTransUnitList()) {
				if (progress("ファイル [" + fileCurrent.getName() + "] (" + (++transUnitCount) + "/" + transUnitTotalCount
						+ ") を処理中...")) {
					break;
				}

				processTransUnit(transUnit);
			}
		}

		if (progress("XLIFF ファイル [" + fileCurrent.getName() + "] を作成／更新中...")) {
			return;
		}

		// 書き出し。
		// TODO 例外処理
		new BlancoXliffSerializer().serialize(xliff, fileCurrent);

		if (progress("XLIFF ファイル [" + fileCurrent.getName() + "] を作成／更新しました。")) {
			return;
		}
	}

	protected void processTransUnit(final BlancoXliffTransUnit transUnit) {
		if (transUnit.getTranslate() == false) {
			// すでに除外されているので、これ以上判断する必要はありません。
			return;
		}
		if (transUnit.getTarget() != null && transUnit.getTarget().getTarget() != null) {
			if (transUnit.getTarget().getTarget().length() > 0) {
				// trans-unit の target に何かしらの値が設定済みのもにについては、
				// 何か意図があって設定されているものとみなし、
				// 除外マーク判定の対象外とします。
				return;
			}
		}

		// もう少し高度な除外判定。
		// TODO 柏原さん: 正規表現による除外判定をこの場所に実装して欲しいです。
		// 希望仕様は以下の通り
		// 1.正規表現は外部ファイルで表現
		// 英文字が 1 文字以下のものは除外します。

		// 翻訳が不要な文字列かどうか判定します。
		if (checkNoTransableString(transUnit.getSource())) {
			transUnit.setTranslate(false);
			return;
		}
	}

	/**
	 * 翻訳が不要な文字列かどうか判定します。
	 *
	 * @param argCheckTarget
	 * @return
	 */
	protected boolean checkNoTransableString(final String argCheckTarget) {
		if (argCheckTarget == null) {
			return true;
		}

		final String trimmedCheckTarget = argCheckTarget.trim();

		// 翻訳不要マーク定義リストをもとに翻訳不要か判断します。
		for (String pattern : markNoTransableList) {
			if (trimmedCheckTarget.matches(pattern)) {

				// 翻訳不要マーク定義パターンに一致した場合は翻訳不要です。
				return true;
			}
		}

		// 翻訳が必要です。
		return false;
	}
}
