/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.cat.core;

import java.util.List;

import benten.cat.translation.core.BentenTmEngine;
import benten.cat.translation.core.BentenTmSearchResult;

/**
 * 翻訳メモリのためのユーティリティ
 */
public class BentenTmUtil {

	/**
	 * 完全一致訳を検索します。
	 *
	 * ・この処理では、文字列の前後の空白は無視して処理を行います。
	 * ・source が完全に一致する翻訳が翻訳メモリにあるかどうか検索します。
	 * ・source が完全一致する翻訳のうち、target が異なる複数の翻訳が無いことを確認します。
	 *
	 * @param tm 翻訳メモリ。
	 * @param source ソース文字列。
	 * @return 適切な完全一致訳が無ければ null。複数 100% マッチする場合も null。1件だけ適切な完全一致訳がある場合にのみ該当する完全一致訳。
	 */
	public static String searchExactMatch(final BentenTmEngine tm, final String source) {
		if (tm == null) {
			throw new IllegalArgumentException("searchExactMatch の tm に null が与えられました。");
		}
		if (source == null) {
			// source が null では、検索する意味がないのですぐに抜けます。
			return null;
		}

		// あいまい検索で結果リストを取得します。
		// 注意: ここでは source そのままを翻訳メモリに渡しています。trim() はかけていません。
		final List<BentenTmSearchResult> matchList = tm.fuzzySearch(source);

		/******
		// 完全一致した翻訳を記憶するためのリスト。
		final List<ITmEntry> exactMatchList = new ArrayList<ITmEntry>();

		for (ITmEntry entry : matchList) {
			// TODO この箇所の処理は、辞書参照時の最適化処理を用いることにより、
			// ある程度のゆとりある完全一致を導出できるようにする予定。

			if (entry.getSrcText().trim().equals(source.trim()) == false) {
				// ソースが完全一致していないので、この翻訳は対象外とします。
				// 注意: TM から戻される翻訳品質という情報については、判断材料から除外しています。
				continue;
			}

			// 入力値とソースが一致しています。これを完全一致した翻訳リストへ記憶します。
			exactMatchList.add(entry);
		}

		if (exactMatchList.size() == 0) {
			// 完全一致訳が 1 件もありませんでした。
			// この source では完全一致訳は 1 件も存在しないため、処理中断します。
			return null;
		}

		// 完全一致訳の 1 件目について正候補として扱います。
		final ITmEntry entry = exactMatchList.get(0);
		final String sourceCandidate = entry.getScore();
		final String targetCandidate = entry.getTranslation();

		// 複数件ヒットした場合の内容確認を実施します。
		for (ITmEntry entryOther : exactMatchList) {
			if (targetCandidate.trim().equals(entryOther.getTranslation().trim()) == false) {
				// 複数の完全一致翻訳のなかで翻訳結果が一致しません。
				// このため、この翻訳結果は完全一致訳としては利用できません。
				// 処理を中断します。
				return null;
			}
		}

		// ここまで通過したので、翻訳は 1 件のみです。
		// ただし、前後の空白は trim して比較しているので、必要に応じて前後に空白を埋め戻します。

		// TODO 柏原さんへ。ここで Pleiades の辞書参照時の最適化の処理を埋めて欲しいです。

		// 前後の空白追加処理。
		return adjustForUntrim(source, sourceCandidate, targetCandidate);
		*****/

		for (BentenTmSearchResult entry : matchList) {
			String target = adjustForUntrim(source, entry.getSource(), entry.getTarget());
			if (target != null) {
				return target;
			}
		}
		return null;
	}

	/**
	 * trim の逆の効果を使って、source と翻訳候補の source, target から target を生成します。
	 *
	 * ☆この処理は Pleiades の処理で代替されることになる予定。
	 *
	 * @param source
	 * @param sourceCandidate
	 * @param targetCandidate
	 * @return target
	 */
	public static String adjustForUntrim(final String source, final String sourceCandidate, final String targetCandidate) {

		/*****
		final int sourceLeftSpaceLen = source.length() - BlancoStringUtil.trimLeft(source).length();
		final int sourceCandidateLeftSpaceLen = sourceCandidate.length()
				- BlancoStringUtil.trimLeft(sourceCandidate).length();
		final int targetCandidateLeftSpaceLen = targetCandidate.length()
				- BlancoStringUtil.trimLeft(targetCandidate).length();
		final int targetLeftSpaceLen = sourceLeftSpaceLen - sourceCandidateLeftSpaceLen + targetCandidateLeftSpaceLen;

		final int sourceRightSpaceLen = source.length() - BlancoStringUtil.trimRight(source).length();
		final int sourceCandidateRightSpaceLen = sourceCandidate.length()
				- BlancoStringUtil.trimRight(sourceCandidate).length();
		final int targetCandidateRightSpaceLen = targetCandidate.length()
				- BlancoStringUtil.trimRight(targetCandidate).length();
		final int targetRightSpaceLen = sourceRightSpaceLen - sourceCandidateRightSpaceLen
				+ targetCandidateRightSpaceLen;
		return getStrings(' ', targetLeftSpaceLen) + targetCandidate.trim() + getStrings(' ', targetRightSpaceLen);
		*****/

		// ソースが一致
		if (source.equals(sourceCandidate)) {
			return targetCandidate;
		}

		JapaneseTranslationString sourceTS = new JapaneseTranslationString(source);
		JapaneseTranslationString tmSourceTS = new JapaneseTranslationString(sourceCandidate);
		JapaneseTranslationString tmTargetTS = new JapaneseTranslationString(targetCandidate);

		// ソースがトリム後に一致
		if (sourceTS.trim().equals(tmSourceTS.trim())) {
			return sourceTS.revert(tmTargetTS.trim());
		}

		// ソースが強制トリム後に一致
		if (sourceTS.trimForce().equals(tmSourceTS.trimForce())) {
			return sourceTS.revert(tmTargetTS.trimForce());
		}
		return null;
	}

	/**
	 * 与えられた char の n 個分の文字列を戻します。
	 *
	 * ☆この処理は Pleiades の処理で代替されることになる予定。
	 *
	 * @param arg
	 * @param length
	 * @return
	protected static String getStrings(char arg, int length) {
		StringBuffer buf = new StringBuffer();
		for (; length-- > 0;) {
			buf.append(arg);
		}
		return buf.toString();
	}
	 */
}
