/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.filter.model;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * HTML ブロックです。
 */
public class HtmlBlock {

	private static boolean IS_DEBUG = Boolean.valueOf(System.getProperty("benten.debug"));
	private static final Pattern enclosedTagPattern = Pattern.compile("(?i)(?s)^(<[^>]+>\\s*)(.+)(\\s*<[^>]+>)$");

	private final Matcher matcher;
	private final boolean isBlockTag;
	private boolean containesHtmlCommentStartTag;
	private boolean containesHtmlCommentEndTag;
	private final String inlineText;

	public HtmlBlock(String input) {
		matcher = enclosedTagPattern.matcher(input);
		isBlockTag = matcher.find();
		if (!isBlockTag) {
			inlineText = null;
			return;
		}

		if (IS_DEBUG) {
			System.out.println("");
			System.out.println("[BLOCK ALL  ] " + compressSpaces(toString()));
			System.out.println("[BLOCK START] " + compressSpaces(startTag()));
			System.out.println("[BLOCK BODY ] " + compressSpaces(body()));
			System.out.println("[BLOCK END  ] " + compressSpaces(endTag()));
		}

		inlineText = trim(body());

		if (IS_DEBUG) {
			System.out.println("---------------------------------");
			System.out.println("[After  ] " + compressSpaces(inlineText));
		}
	}

	public boolean isBlockTag() {
		return isBlockTag;
	}

	public boolean equalsStartEndTag() {
		if (startTagName().equals(endTagName())) {
			String pattern = "(?i)(?s).*<" + startTagName() + "(\\s|>).*";
			return !body().matches(pattern);
		}
		return false;
	}

	public String startTag() {
		return matcher.group(1);
	}

	public String startTagName() {
		return getTagName(startTag());
	}

	public String body() {
		return matcher.group(2);
	}

	public String endTag() {
		return matcher.group(3);
	}

	public String endTagName() {
		return getTagName(endTag());
	}

	public int endTagIndex() {
		return matcher.start(3);
	}

	private String getTagName(String tag) {
		return tag.trim().replaceAll("([/<>]|\\s\\w.+)", "");
	}

	public boolean containesHtmlCommentStartTag() {
		return containesHtmlCommentStartTag;
	}

	public boolean containesHtmlCommentEndTag() {
		return containesHtmlCommentEndTag;
	}

	@Override
	public String toString() {
		return startTag() + body() + endTag();
	}

	public String getInlineText() {
		return inlineText;
	}

	private String trim(String text) {
		// 前後のブロック・タグ除去
		String s = HtmlBlockMatcher.trimBlockTag(text);

		// HTML コメント処理
		s = removeHtmlComment(s);

		// 読み飛ばし判定
		if (isIgnore(s)) {
			return "";
		}
		// 全体が a タグでない場合 (a タグが片側だけになるのを防止)
		if (!s.matches("(?i)(?s)<a\\s.+?>.*</a>")) {

			// 前後のインライン・タグのうち、タグ内のテキストが空の場合は除去
			s = HtmlBlockMatcher.trimTag(s, "<[^</]+>\\s*</[^<]+>");
		}
		// 前後の対応するインライン・タグを除去 (属性に title や alt がある場合は除去しない)
		HtmlBlock block = new HtmlBlock(s);
		if (block.isBlockTag() && block.equalsStartEndTag()) {
			if (!needsTranslationAttribute(block.startTag())) {
				s = block.body();
			}
		}

		if (s.equals(text)) {
			return s;
		}
		return trim(s); // 再帰
	}

	private String removeHtmlComment(String s) {
		final String HTML_COMMENT_START = "<!--";
		final String HTML_COMMENT_END = "-->";

		if (s.contains(HTML_COMMENT_START) && s.contains(HTML_COMMENT_END)) {
			String pattern = "(?i)(?s)" + HTML_COMMENT_START + ".*" + HTML_COMMENT_END;
			s = s.replaceAll(pattern, "");
		}
		if (s.contains(HTML_COMMENT_START)) {
			String pattern = "(?i)(?s)" + HTML_COMMENT_START + ".*";
			s = s.replaceFirst(pattern, "");
			containesHtmlCommentStartTag = true;
		} else if (s.contains(HTML_COMMENT_END)) {
			String pattern = "(?i)(?s)" + ".*?" + HTML_COMMENT_END;
			s = s.replaceFirst(pattern, "");
			containesHtmlCommentEndTag = true;
		}
		return s;
	}

	private boolean isIgnore(String s) {
		// 1 文字の場合
		if (s.length() == 1) {
			return true;
		}
		// 全体が 1 つのタグ (テキスト部が無い) 、かつ翻訳が必要な属性が無い場合
		if (s.matches("(?i)(?s)<[^<]+>") && !needsTranslationAttribute(s)) {
			return true;
		}
		// 全体が属性の無い 1 つのタグ (つまりテキスト部が無い) の場合
		if (s.matches("(?i)(?s)^<[^<\\s]+>$")) {
			return true;
		}
		return false;
	}

	private boolean needsTranslationAttribute(String text) {
		return text.matches("(?i)(?s).+?\\s(title|alt)=\".+");
	}

	private String compressSpaces(String s) {
		return s.replaceAll("\\s+", " ").replaceAll(" {2,}", " ");
	}
}
