/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.filter.model;

import java.util.LinkedList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * 分割された文のリストです。
 */
@SuppressWarnings("serial")
public class SentenceSplittedList extends LinkedList<String> {

	/**
	 * コンストラクタ。
	 * @param text テキスト
	 */
	public SentenceSplittedList(String text) {

		// 単純に分割する
		LinkedList<String> tempList = split(text);

		// 分割したものを検査し、このインスタンスのリストに格納
		makeWithValidate(tempList);
	}

	/**
	 * テキストを英語の句点（「. 」「: 」）で分割します。
	 * @param text テキスト
	 * @return テキストを分割した文を要素に持つリスト
	 */
	protected LinkedList<String> split(String text) {
		Matcher matcher = Pattern.compile(".+?(\\.|:)\\s+").matcher(text);
		LinkedList<String> list = new LinkedList<String>();
		while (matcher.find()) {
			StringBuffer sb = new StringBuffer();
			matcher.appendReplacement(sb, Matcher.quoteReplacement(matcher.group()));
			list.add(sb.toString());
		}
		StringBuffer sb = new StringBuffer();
		matcher.appendTail(sb);
		list.add(sb.toString());
		return list;
	}

	/**
	 * リストの内容が文として妥当か判定し、リストを作成します。
	 * 妥当でない場合は、連結されます。
	 * @param list 分割されたリスト
	 */
	protected void makeWithValidate(LinkedList<String> list) {
		String prev = "";
		while (list.size() > 0) {
			String s = prev + list.removeFirst();
			boolean isValid = true;

			// 囲み文字の対応を検査
			isValid = isValidSurroundChar(s, '<', '>') && isValid;
			isValid = isValidSurroundChar(s, '(', ')') && isValid;

			// インライン・タグの対応を検査
			isValid = isValidSurroundTag(s, "i") && isValid;
			isValid = isValidSurroundTag(s, "font") && isValid;

			// 正しい場合はリストに追加、不正な場合は次の要素と連結
			if (isValid) {
				add(s);
				prev = "";
			} else {
				prev = s;
			}
		}
		if (!prev.equals("")) {
			add(prev);
		}
	}

	/**
	 * 括弧などの囲み文字の対が正しいか判定します。
	 * @param text テキスト
	 * @param c1 囲み文字の開始文字
	 * @param c2 囲み文字の終了文字
	 * @return 正しい場合は true
	 */
	protected boolean isValidSurroundChar(String text, char c1, char c2) {
		int c1Count = 0;
		int c2Count = 0;
		for (char c : text.toCharArray()) {
			if (c == c1) {
				c1Count++;
			} else if (c == c2) {
				c2Count++;
			}
		}
		return c1Count == c2Count;
	}

	/**
	 * 開始終了タグが対で含まれているか判定します。
	 * @param text テキスト
	 * @param tag タグ
	 * @return 対になっている場合は true
	 */
	protected boolean isValidSurroundTag(String text, String tag) {
		int startTagCount = countMatchs(text, "<" + tag + ">");
		int endTagCount = countMatchs(text, "</" + tag + "(|\\s+[^>]*)>");
		return startTagCount == endTagCount;
	}

	/**
	 * パターンが出現する回数をカウントします。
	 * @param text テキスト
	 * @param pattern 出現パターン
	 * @return 出現回数
	 */
	private int countMatchs(String text, String pattern) {
		Matcher startTag = Pattern.compile("(?i)(?s)" + pattern).matcher(text);
		int count = 0;
		while (startTag.find()) {
			count++;
		}
		return count;
	}
}
