/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.tmx.core;

import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;

import benten.core.io.Files;
import benten.core.io.TmxFileFilter;
import benten.core.model.Tmx;
import benten.core.model.Xliff;
import benten.twa.tmx.core.valueobject.BentenExtractTmxFragmentProcessInput;
import blanco.tmx.valueobject.BlancoTmxTu;
import blanco.tmx.valueobject.BlancoTmxTuv;
import blanco.xliff.BlancoXliffUtil;
import blanco.xliff.valueobject.BlancoXliffFile;
import blanco.xliff.valueobject.BlancoXliffTransUnit;

public class BentenExtractTmxFragmentProcessImpl implements BentenExtractTmxFragmentProcess {
	private final String nowDate = new SimpleDateFormat("yyyyMMdd").format(new Date());

	/**
	 * クラスをインスタンス化して処理を実行する際のエントリポイントです。
	 *
	 * @param input 処理の入力パラメータ。
	 * @return 処理の実行結果。
	 * @throws IOException 入出力例外が発生した場合。
	 * @throws IllegalArgumentException 入力値に不正が見つかった場合。
	 */
	public int execute(final BentenExtractTmxFragmentProcessInput input) throws IOException, IllegalArgumentException {
		final Xliff inXliff = Xliff.loadInstance(new File(input.getXlifffile()));
		final Tmx outTmx = Tmx.newInstance(createFile(new File(input.getXlifffile()), new File(input.getTmxdir()),
				input.getIncludedate()));

		int transUnitCount = 0;
		int transUnitTotalCount = 0;
		for (BlancoXliffFile inFile : inXliff.getFileList()) {
			transUnitTotalCount += inFile.getBody().getTransUnitList().size();
			for (BlancoXliffTransUnit inUnit : inFile.getBody().getTransUnitList()) {
				if (progress("ファイル [" + input.getXlifffile() + "] (" + (++transUnitCount) + "/" + transUnitTotalCount
						+ ") を処理中...")) {
					break;
				}

				if (inUnit.getTranslate() == false) {
					continue;
				}
				if (inUnit.getSource() == null || inUnit.getSource().length() == 0) {
					continue;
				}
				if (inUnit.getTarget() == null || inUnit.getTarget().getTarget() == null
						|| inUnit.getTarget().getTarget().length() == 0) {
					continue;
				}

				// TMX反映除外のチェック。
				if (BlancoXliffUtil.isXTmOmit(inUnit)) {
					continue;
				}

				final BlancoTmxTu tu = new BlancoTmxTu();
				outTmx.getTuList().add(tu);
				{
					BlancoTmxTuv tuv = new BlancoTmxTuv();
					tu.getTuvList().add(tuv);
					tuv.setLang(inFile.getSourceLanguage());
					tuv.setSeg(inUnit.getSource());
				}
				{
					BlancoTmxTuv tuv = new BlancoTmxTuv();
					tu.getTuvList().add(tuv);
					tuv.setLang(inFile.getTargetLanguage());
					tuv.setSeg(inUnit.getTarget().getTarget());
				}
			}
		}

		if (progress("TMX ファイル [" + input.getXlifffile() + "] を作成中...")) {
			return 0;
		}

		outTmx.save();

		if (progress("TMX ファイル [" + input.getXlifffile() + "] を作成しました。")) {
			return 0;
		}

		return 0;
	}

	/**
	 * 処理の中でアイテムが処理されるたびに進捗報告としてコールバックします。
	 *
	 * @param argProgressMessage 現在処理しているアイテムに関するメッセージ。
	 * @return 処理をそのまま継続する場合は false。処理中断をリクエストしたい場合は true。
	 */
	public boolean progress(final String argProgressMessage) {
		return false;
	}

	protected File createFile(File targetFile, File toDir, boolean includeDate) {
		final StringBuilder name = new StringBuilder();
		name.append(Files.baseName(targetFile));
		if (includeDate) {
			name.append("-" + nowDate);
		}
		return TmxFileFilter.appendExtension(new File(toDir, name.toString()));
	}
}
