/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.tmx.core;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import benten.core.io.Files;
import benten.core.io.TmxFileFilter;
import benten.twa.tmx.core.valueobject.BentenMergeTmxProcessInput;
import benten.twa.tmx.handler.MergeTmxHandler.MergeTmxStatus;
import blanco.tmx.BlancoTmxParser;
import blanco.tmx.BlancoTmxSerializer;
import blanco.tmx.BlancoTmxUtil;
import blanco.tmx.valueobject.BlancoTmx;
import blanco.tmx.valueobject.BlancoTmxTu;

public class BentenMergeTmxProcessImpl implements BentenMergeTmxProcess {

	/**
	 * クラスをインスタンス化して処理を実行する際のエントリポイントです。
	 *
	 * @param input 処理の入力パラメータ。
	 * @return 処理の実行結果。
	 * @throws IOException 入出力例外が発生した場合。
	 * @throws IllegalArgumentException 入力値に不正が見つかった場合。
	 */
	public int execute(final BentenMergeTmxProcessInput input) throws IOException, IllegalArgumentException {
		if (progress("ファイル [" + input.getSourcefile() + "]を読み込み中...")) {
			return 0;
		}
		final BlancoTmx tmxIn = new BlancoTmxParser().parse(new File(input.getSourcefile()));

		if (progress("ファイル [" + input.getTargetfile() + "]を読み込み中...")) {
			return 0;
		}
		final BlancoTmx tmxOut = new BlancoTmxParser().parse(new File(input.getTargetfile()));

		if (progress("マップを作成中...")) {
			return 0;
		}
		final Map<String, String> mapTmxOut = new HashMap<String, String>(16384);
		for (BlancoTmxTu tu : tmxOut.getBody().getTuList()) {
			final String en = BlancoTmxUtil.getTuvByLang(tu, "EN").getSeg();
			final String ja = BlancoTmxUtil.getTuvByLang(tu, "JA").getSeg();
			mapTmxOut.put("en:[" + en + "], ja:[" + ja + "]", "exist");
		}

		if (progress("マップから引き当て中...")) {
			return 0;
		}
		for (BlancoTmxTu tu : tmxIn.getBody().getTuList()) {
			final String en = BlancoTmxUtil.getTuvByLang(tu, "EN").getSeg();
			final String ja = BlancoTmxUtil.getTuvByLang(tu, "JA").getSeg();
			if (mapTmxOut.get("en:[" + en + "], ja:[" + ja + "]") != null) {
				// すでに一致する登録が存在。
				// TODO 現時点では、一致する登録は音も無く無視しています。
			} else {
				tmxOut.getBody().getTuList().add(tu);
			}
		}

		if (progress("TMX ファイル [" + input.getTargetfile() + "] を更新中...")) {
			return 0;
		}

		new BlancoTmxSerializer().serialize(tmxOut, new File(input.getTargetfile()));

		if (progress("TMX ファイル [" + input.getTargetfile() + "] を更新しました。")) {
			return 0;
		}

		return 0;
	}

	/**
	 * 処理の中でアイテムが処理されるたびに進捗報告としてコールバックします。
	 *
	 * @param argProgressMessage 現在処理しているアイテムに関するメッセージ。
	 * @return 処理をそのまま継続する場合は false。処理中断をリクエストしたい場合は true。
	 */
	public boolean progress(final String argProgressMessage) {
		return false;
	}

	protected File createFile(MergeTmxStatus status) {
		StringBuilder name = new StringBuilder();
		name.append(Files.baseName(status.targetFile));
		return TmxFileFilter.appendExtension(new File(status.toFile, name.toString()));
	}
}
