/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.xliff.core;

import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;

import benten.core.io.Files;
import benten.core.io.XliffFileFilter;
import benten.core.model.Xliff;
import benten.twa.xliff.core.valueobject.BentenExtractXliffFragmentProcessInput;
import blanco.xliff.valueobject.BlancoXliffFile;
import blanco.xliff.valueobject.BlancoXliffTarget;
import blanco.xliff.valueobject.BlancoXliffTransUnit;

public class BentenExtractXliffFragmentProcessImpl implements BentenExtractXliffFragmentProcess {
	private final String nowDate = new SimpleDateFormat("yyyyMMdd").format(new Date());

	/**
	 * クラスをインスタンス化して処理を実行する際のエントリポイントです。
	 *
	 * @param input 処理の入力パラメータ。
	 * @return 処理の実行結果。
	 * @throws IOException 入出力例外が発生した場合。
	 * @throws IllegalArgumentException 入力値に不正が見つかった場合。
	 */
	public int execute(final BentenExtractXliffFragmentProcessInput input) throws IOException, IllegalArgumentException {
		if (progress("ファイル [" + input.getSourcexlifffile() + "] を処理中...")) {
			return 0;
		}
		Xliff inXliff = Xliff.loadInstance(new File(input.getSourcexlifffile()));

		int outXliffIndex = 0;
		Xliff outXliff = Xliff.newInstance(createFile(input, ++outXliffIndex));

		int transUnitCount = 0;
		int transUnitTotalCount = 0;
		for (BlancoXliffFile inFile : inXliff.getFileList()) {
			transUnitTotalCount += inFile.getBody().getTransUnitList().size();

			BlancoXliffFile outFile = createBlancoXliffFile(outXliff, inFile);

			for (BlancoXliffTransUnit inUnit : inFile.getBody().getTransUnitList()) {
				if (progress("ファイル [" + new File(input.getSourcexlifffile()).getName() + "] (" + (++transUnitCount)
						+ "/" + transUnitTotalCount + ") を処理中...")) {
					break;
				}

				if (input.getExtractbystate() != null && input.getExtractbystate().length() > 0) {
					BlancoXliffTarget target = inUnit.getTarget();
					String state = (target == null) ? "" : target.getState();
					state = (state == null) ? "" : state;
					if (!input.getExtractbystate().equals(state)) {
						continue;
					}
				}
				outFile.getBody().getTransUnitList().add(inUnit);
				if (input.getLimitsize() > 0 && outXliff.getAllTransUnitList().size() == input.getLimitsize()) {
					outXliff.save();
					outXliff = Xliff.newInstance(createFile(input, ++outXliffIndex));
					outFile = createBlancoXliffFile(outXliff, inFile);
				}
			}
		}

		if (progress("XLIFF ファイルを作成中...")) {
			return 0;
		}

		outXliff.save();

		if (progress("XLIFF ファイルを作成しました。")) {
			return 0;
		}

		return 0;
	}

	/**
	 * 処理の中でアイテムが処理されるたびに進捗報告としてコールバックします。
	 *
	 * @param argProgressMessage 現在処理しているアイテムに関するメッセージ。
	 * @return 処理をそのまま継続する場合は false。処理中断をリクエストしたい場合は true。
	 */
	public boolean progress(final String argProgressMessage) {
		return false;
	}

	protected File createFile(final BentenExtractXliffFragmentProcessInput input, int fileIndex) {
		StringBuilder name = new StringBuilder();
		name.append(Files.baseName(new File(input.getSourcexlifffile())));
		if (input.getLimitsize() > 0) {
			name.append("-" + fileIndex);
		}
		if (input.getIncludedate()) {
			name.append("-" + nowDate);
		}
		return XliffFileFilter.appendExtension(new File(input.getTargetxliffdir(), name.toString()));
	}

	protected BlancoXliffFile createBlancoXliffFile(Xliff outXliff, BlancoXliffFile inFile) {
		BlancoXliffFile outFile = outXliff.createBlancoXliffFile();
		outFile.setSourceLanguage(inFile.getSourceLanguage());
		outFile.setTargetLanguage(inFile.getTargetLanguage());
		outFile.setDatatype(inFile.getDatatype());
		outFile.setHeader(inFile.getHeader());
		return outFile;
	}
}
