/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.xliff.dialogs;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import benten.core.io.XliffFileFilter;
import benten.core.model.Xliff;
import benten.twa.ui.dialogs.AbstractWorkflowDialog;
import benten.twa.ui.fields.FileTextField;
import benten.twa.ui.fields.ResourceFolderTextField;
import benten.twa.xliff.handler.ExtractXliffFragmentHandler.ExtractXliffFragmentStatus;
import benten.ui.UiStatus;
import benten.ui.UiStatusException;

/**
 * XLIFF フラグメント抽出ダイアログです。
 */
public class ExtractXliffFragmentDialog extends AbstractWorkflowDialog {

	private final ExtractXliffFragmentControls controls = new ExtractXliffFragmentControls();

	public static class ExtractXliffFragmentControls {
		public FileTextField fromXliffFileText;
		public ResourceFolderTextField toDirText;
		public Button stateCheck;
		public Combo stateCombo;
		public Button countCheck;
		public Text countText;
		public Button includeDateCheck;
	}

	public ExtractXliffFragmentDialog(Shell parentShell, IStructuredSelection selection) {
		super(parentShell, selection, "XLIFF フラグメントの抽出");
	}

	@Override
	public void createControls(Composite composite) {
		composite.setLayout(new GridLayout(3, false));

		controls.fromXliffFileText = new FileTextField(composite, "対象ファイル", XliffFileFilter.DEFAULT_EXTENSION);
		controls.fromXliffFileText.addModifyListener(this);
		controls.fromXliffFileText.setText(getSelectionResourceString());

		controls.toDirText = new ResourceFolderTextField(composite, "出力先");
		controls.toDirText.addModifyListener(this);
		if (getSelectionResource() != null) {
			controls.toDirText.setText(getSelectionResource().getProject().getName() + "/xliff.fragment");
		}

		Group group = new Group(composite, SWT.NULL);
		group.setText("抽出方法");
		GridData howToExtractGrid = new GridData(GridData.FILL_HORIZONTAL);
		howToExtractGrid.horizontalSpan = 2;
		group.setLayoutData(howToExtractGrid);
		group.setLayout(new GridLayout(2, false));

		controls.stateCheck = new Button(group, SWT.CHECK);
		controls.stateCheck.setText("状態で抽出");
		controls.stateCheck.setSelection(true);
		controls.stateCheck.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				boolean checked = ((Button) e.widget).getSelection();
				controls.stateCombo.setEnabled(checked);
			}
		});

		controls.stateCombo = new Combo(group, SWT.READ_ONLY);
		controls.stateCombo.setItems(Xliff.getStateItems());
		controls.stateCombo.setLayoutData(new GridData());

		controls.countCheck = new Button(group, SWT.CHECK);
		controls.countCheck.setText("件数で区切って抽出");
		controls.countCheck.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				boolean checked = ((Button) e.widget).getSelection();
				controls.countText.setEnabled(checked);
			}
		});

		controls.countText = new Text(group, SWT.BORDER);
		controls.countText.setLayoutData(new GridData());
		controls.countText.setEnabled(false);
		controls.countText.addModifyListener(this);

		Label spacer = new Label(composite, SWT.NULL);
		spacer.setText("");

		Group group1 = new Group(composite, SWT.NULL);
		group1.setText("出力ファイル名オプション");
		GridData outputFileNameOptionGrid = new GridData(GridData.FILL_HORIZONTAL);
		outputFileNameOptionGrid.horizontalSpan = 2;
		group1.setLayoutData(outputFileNameOptionGrid);
		group1.setLayout(new GridLayout(1, false));

		controls.includeDateCheck = new Button(group1, SWT.CHECK);
		controls.includeDateCheck.setText("ファイル名に年月日を含める");
		controls.includeDateCheck.setSelection(true);
	}

	@Override
	public IStatus getResultStatus() {
		ExtractXliffFragmentStatus status = new ExtractXliffFragmentStatus();
		status.fromXliffFile = controls.fromXliffFileText.getFile();
		status.toDir = controls.toDirText.getFile();
		if (controls.stateCheck.getSelection()) {
			status.state = controls.stateCombo.getText();
		}
		if (controls.countCheck.getSelection()) {
			status.sizeLimit = Integer.parseInt(controls.countText.getText());
		}
		status.includeDate = controls.includeDateCheck.getSelection();
		status.setProject(controls.toDirText.getResource());
		return status;
	}

	@Override
	public IStatus validate() throws UiStatusException {
		validatePath(controls.fromXliffFileText);
		validatePath(controls.toDirText);
		if (controls.countCheck != null && controls.countCheck.getSelection()) {
			try {
				int count = Integer.parseInt(controls.countText.getText());
				if (count < 1) {
					throw new UiStatusException("件数は 1 以上の数値を入力してください。");
				}
			} catch (RuntimeException e) {
				throw new UiStatusException("件数に数値を入力してください。");
			}
		}
		return UiStatus.OK_STATUS;
	}
}
